<#
Script written Oct/06/2017 | Login VSI, Inc.
CREDIT FOR THE CONNECT-MSTSC FUNCTION GOES TO JAAP BRASSER. PLEASE SEE THE FOLLOWING PAGE FOR MORE INFO: https://gallery.technet.microsoft.com/Connect-Mstsc-Open-RDP-2064b10b

1. This script will restart specified launcher hosts then reconnect to them with specified launcher user credentials via mstsc. It's recommended launchers are restarted after each test as a Login VSI best practice

2. Ensure launcher user has read, write, and modify access to the Login VSI share.

3. Ensure the launcher user will run a logon script that will remotely run agent.exe (the launcher agent) from the Login VSI share. For example, a one-line batch script will work, such as "CALL <unc_path_to_agent.exe>" in the AD for the user, or in the start menu startup directory

4. Ensure the host running this script has permissions to remotely reboot and remote desktop to the launcher hosts

5. Configure $launcherUsername variable in this script in between double quotes. Might need to be in the domain\username format or just username

6. Configure $launcherPassword variable in this script in between double quotes. For enhanced password security, ctrl+f this script for the usage examples with "ConvertTo-SecureString". Credit to Jaap Brasser.

7. Congigure $csvFile variable in this scrpt in between double quotes. This is the full path to the launcher.csv file. Within the file, include each launcher hostname on its own line. The csv file only needs to be one column. Don't include a column header/title.

8. Depending on execution policies, might need to run this script in the command-line format of "PowerShell.exe -ExecutionPolicy Bypass -File <path_to_script>"
#>

$launcherUsername = ""
$launcherPassword = ""
$csvFile = ""

Write-Host `r`n

If (!($psISE)){"Press any key once all the launchers for the next test have been added to the defined launchers csv file path in the script, and the csv file is saved. Don't include a column header/name in the CSV file.";[void][System.Console]::ReadKey($true)}
$csv = Import-Csv $csvFile -Header launchers

Write-Host `r`n
If (!($psISE)){"Press any key when the Launcher Workflow feature is disabled in the Start Test Wizard of the Login VSI Management Console";[void][System.Console]::ReadKey($true)}

Write-Host `r`n
Write-Host "Once all the launchers listed in the csv are ready to be restarted, press any key" `r`n
If (!($psISE)){[void][System.Console]::ReadKey($true)}

foreach ($line in $csv) {
	 Write-Host "Restarting $($line.launchers)"
	 Restart-Computer -ComputerName ($line.launchers) -Force
}

Function Connect-Mstsc {
<#   
.SYNOPSIS   
Function to connect an RDP session without the password prompt
    
.DESCRIPTION 
This function provides the functionality to start an RDP session without having to type in the password
	
.PARAMETER ComputerName
This can be a single computername or an array of computers to which RDP session will be opened

.PARAMETER User
The user name that will be used to authenticate

.PARAMETER Password
The password that will be used to authenticate

.PARAMETER Credential
The PowerShell credential object that will be used to authenticate against the remote system

.PARAMETER Admin
Sets the /admin switch on the mstsc command: Connects you to the session for administering a server

.PARAMETER MultiMon
Sets the /multimon switch on the mstsc command: Configures the Remote Desktop Services session monitor layout to be identical to the current client-side configuration 

.PARAMETER FullScreen
Sets the /f switch on the mstsc command: Starts Remote Desktop in full-screen mode

.PARAMETER Public
Sets the /public switch on the mstsc command: Runs Remote Desktop in public mode

.PARAMETER Width
Sets the /w:<width> parameter on the mstsc command: Specifies the width of the Remote Desktop window

.PARAMETER Height
Sets the /h:<height> parameter on the mstsc command: Specifies the height of the Remote Desktop window

.NOTES   
Name:        Connect-Mstsc
Author:      Jaap Brasser
DateUpdated: 2016-10-28
Version:     1.2.5
Blog:        http://www.jaapbrasser.com

.LINK
http://www.jaapbrasser.com

.EXAMPLE   
. .\Connect-Mstsc.ps1
    
Description 
-----------     
This command dot sources the script to ensure the Connect-Mstsc function is available in your current PowerShell session

.EXAMPLE   
Connect-Mstsc -ComputerName server01 -User contoso\jaapbrasser -Password (ConvertTo-SecureString 'supersecretpw' -AsPlainText -Force)

Description 
-----------     
A remote desktop session to server01 will be created using the credentials of contoso\jaapbrasser

.EXAMPLE   
Connect-Mstsc server01,server02 contoso\jaapbrasser (ConvertTo-SecureString 'supersecretpw' -AsPlainText -Force)

Description 
-----------     
Two RDP sessions to server01 and server02 will be created using the credentials of contoso\jaapbrasser

.EXAMPLE   
server01,server02 | Connect-Mstsc -User contoso\jaapbrasser -Password (ConvertTo-SecureString 'supersecretpw' -AsPlainText -Force) -Width 1280 -Height 720

Description 
-----------     
Two RDP sessions to server01 and server02 will be created using the credentials of contoso\jaapbrasser and both session will be at a resolution of 1280x720.

.EXAMPLE   
server01,server02 | Connect-Mstsc -User contoso\jaapbrasser -Password (ConvertTo-SecureString 'supersecretpw' -AsPlainText -Force) -Wait

Description 
-----------     
RDP sessions to server01 will be created, once the mstsc process is closed the session next session is opened to server02. Using the credentials of contoso\jaapbrasser and both session will be at a resolution of 1280x720.

.EXAMPLE   
Connect-Mstsc -ComputerName server01:3389 -User contoso\jaapbrasser -Password (ConvertTo-SecureString 'supersecretpw' -AsPlainText -Force) -Admin -MultiMon

Description 
-----------     
A RDP session to server01 at port 3389 will be created using the credentials of contoso\jaapbrasser and the /admin and /multimon switches will be set for mstsc

.EXAMPLE   
Connect-Mstsc -ComputerName server01:3389 -User contoso\jaapbrasser -Password (ConvertTo-SecureString 'supersecretpw' -AsPlainText -Force) -Public

Description 
-----------     
A RDP session to server01 at port 3389 will be created using the credentials of contoso\jaapbrasser and the /public switches will be set for mstsc

.EXAMPLE
Connect-Mstsc -ComputerName 192.168.1.10 -Credential $Cred

Description 
-----------     
A RDP session to the system at 192.168.1.10 will be created using the credentials stored in the $cred variable.

.EXAMPLE   
Get-AzureVM | Get-AzureEndPoint -Name 'Remote Desktop' | ForEach-Object { Connect-Mstsc -ComputerName ($_.Vip,$_.Port -join ':') -User contoso\jaapbrasser -Password (ConvertTo-SecureString 'supersecretpw' -AsPlainText -Force) }

Description 
-----------     
A RDP session is started for each Azure Virtual Machine with the user contoso\jaapbrasser and password supersecretpw

.EXAMPLE
PowerShell.exe -Command "& {. .\Connect-Mstsc.ps1; Connect-Mstsc server01 contoso\jaapbrasser (ConvertTo-SecureString 'supersecretpw' -AsPlainText -Force) -Admin}"

Description
-----------
An remote desktop session to server01 will be created using the credentials of contoso\jaapbrasser connecting to the administrative session, this example can be used when scheduling tasks or for batch files.
#>
    [cmdletbinding(SupportsShouldProcess,DefaultParametersetName='UserPassword')]
    param (
        [Parameter(Mandatory=$true,
            ValueFromPipeline=$true,
            ValueFromPipelineByPropertyName=$true,
            Position=0)]
        [Alias('CN')]
            [string[]]     $ComputerName,
        [Parameter(ParameterSetName='UserPassword',Mandatory=$true,Position=1)]
        [Alias('U')] 
            [string]       $User,
        [Parameter(ParameterSetName='UserPassword',Mandatory=$true,Position=2)]
        [Alias('P')] 
            [string]       $Password,
        [Parameter(ParameterSetName='Credential',Mandatory=$true,Position=1)]
        [Alias('C')]
            [PSCredential] $Credential,
        [Alias('A')]
            [switch]       $Admin,
        [Alias('MM')]
            [switch]       $MultiMon,
        [Alias('F')]
            [switch]       $FullScreen,
        [Alias('Pu')]
            [switch]       $Public,
        [Alias('W')]
            [int]          $Width,
        [Alias('H')]
            [int]          $Height,
        [Alias('WT')]
            [switch]       $Wait
    )

    begin {
        [string]$MstscArguments = ''
        switch ($true) {
            {$Admin}      {$MstscArguments += '/admin '}
            {$MultiMon}   {$MstscArguments += '/multimon '}
            {$FullScreen} {$MstscArguments += '/f '}
            {$Public}     {$MstscArguments += '/public '}
            {$Width}      {$MstscArguments += "/w:$Width "}
            {$Height}     {$MstscArguments += "/h:$Height "}
        }

        if ($Credential) {
            $User     = $Credential.UserName
            $Password = $Credential.GetNetworkCredential().Password
        }
    }
    process {
        foreach ($Computer in $ComputerName) {
            $ProcessInfo = New-Object System.Diagnostics.ProcessStartInfo
            $Process = New-Object System.Diagnostics.Process
            
            # Remove the port number for CmdKey otherwise credentials are not entered correctly
            if ($Computer.Contains(':')) {
                $ComputerCmdkey = ($Computer -split ':')[0]
            } else {
                $ComputerCmdkey = $Computer
            }

            $ProcessInfo.FileName    = "$($env:SystemRoot)\system32\cmdkey.exe"
            $ProcessInfo.Arguments   = "/generic:TERMSRV/$ComputerCmdkey /user:$User /pass:$($Password)"
            $ProcessInfo.WindowStyle = [System.Diagnostics.ProcessWindowStyle]::Hidden
            $Process.StartInfo = $ProcessInfo
            if ($PSCmdlet.ShouldProcess($ComputerCmdkey,'Adding credentials to store')) {
                [void]$Process.Start()
            }

            $ProcessInfo.FileName    = "$($env:SystemRoot)\system32\mstsc.exe"
            $ProcessInfo.Arguments   = "$MstscArguments /v $Computer"
            $ProcessInfo.WindowStyle = [System.Diagnostics.ProcessWindowStyle]::Normal
            $Process.StartInfo       = $ProcessInfo
            if ($PSCmdlet.ShouldProcess($Computer,'Connecting mstsc')) {
                [void]$Process.Start()
                if ($Wait) {
                    $null = $Process.WaitForExit()
                }       
            }
        }
    }
}

Write-Host `r`n
Write-Host "Once all launcher hosts should be rebooted and idling at the Windows account login screen, press any key" `r`n
If (!($psISE)){[void][System.Console]::ReadKey($true)}

Write-Host `r`n
$waitInbetweenMstscCommands = Read-Host -Prompt 'Enter how many seconds to wait (as a plain integer) before executing new mstsc commands to each launcher. i.e. a Remote Desktop Session Host might report in as busy if performing too many simultaneous operations. If don''t need to wait then use the number 0'
Write-Host `r`n

foreach ($line in $csv) {
	 Write-Host "RDPing to $($line.launchers)"
	 Connect-Mstsc -User $launcherUsername -Password $launcherPassword -Width 1024 -Height 768 -ComputerName ($line.launchers)
	 Start-Sleep -s $waitInbetweenMstscCommands
}

Write-Host `r`n
Write-Host "1. Now go to to Launcher check page in the Start Test Wizard in the Login VSI Management Console."
Write-Host "2. The launchers should show up as Ready status"
If (!($psISE)){"Press any key to continue";[void][System.Console]::ReadKey($true)}
Write-Host `r`n
Write-Host `r`n
Write-Host "Start the test. Once the test is in progress (Dashboard can be seen), the mstsc windows for the launchers can be safely closed out of; the test should continue"
Write-Host `r`n
If (!($psISE)){"Press any key to exit the script";[void][System.Console]::ReadKey($true)}
Write-Host `r`n